"""Generate a status line for swaybar."""

import sys
import toml
from pathlib import Path
from argparse import ArgumentParser
from .loop import start
from .config import config
from .modules import Modules
from .logging import logger, configure as configure_logging
from .env import bin_name, config_home, environ_path, environ_paths


def parse_args():
    p = ArgumentParser(description=__doc__)

    p.add_argument(
        "-c",
        "--config-file",
        metavar="FILE",
        type=Path,
        help="override configuration file",
    )

    p.add_argument(
        "-C",
        "--config-dir",
        metavar="DIRECTORY",
        type=Path,
        help="override configuration directory",
    )

    p.add_argument(
        "-I",
        "--include",
        action="append",
        metavar="DIRECTORY",
        type=Path,
        help="include additional modules package",
    )

    p.add_argument(
        "-i",
        "--interval",
        type=float,
        metavar="SECONDS",
        help="override default update interval",
    )

    p.add_argument(
        "--no-click-events",
        dest="click_events",
        action="store_false",
        help="disable click events",
    )

    p.add_argument(
        "-L",
        "--log-level",
        metavar="LEVEL",
        choices=["debug", "info", "warning", "error", "critical"],
        help="override default logging minimum severity level",
    )

    p.add_argument(
        "-l",
        "--log-file",
        metavar="FILE",
        type=Path,
        help="output logging to %(metavar)s",
    )

    p.add_argument(
        "--syslog",
        action="store_true",
        help="output logging to syslog",
    )

    return p.parse_args()


def parse_config(args):
    config_dir = args.config_dir or environ_path(
        "SWAYSTATUS_CONFIG_DIR", config_home / bin_name
    )
    config_file = args.config_file or environ_path(
        "SWAYSTATUS_CONFIG_FILE", config_dir / "config.toml"
    )

    if config_file.is_file():
        config.update(toml.loads(open(config_file).read()))

    config["include"] = (
        (args.include or [])
        + [config_dir / "modules"]
        + [Path(d).expanduser() for d in config.get("include", [])]
        + environ_paths("SWAYSTATUS_MODULE_PATH")
    )

    if args.interval:
        config["interval"] = args.interval

    if not args.click_events:
        config["click_events"] = False

    return config


def load_elements(order, include, settings):
    elements = []
    modules = Modules(include)

    for key in order:
        ids = key.split(":", maxsplit=1)
        ids.append(None)

        name, instance = ids[:2]

        module = modules.find(name)

        element_settings = {"name": name, "instance": instance}
        element_settings.update(settings.get(name, {}).copy())
        element_settings.update(settings.get(key, {}).copy())

        logger.info(f"Loaded module from file: {module.__file__}")
        logger.debug(f"Initializing module: {element_settings!r}")

        elements.append(module.Element(**element_settings))

    return elements


def main():
    args = parse_args()

    configure_logging(level=args.log_level, file=args.log_file, syslog=args.syslog)

    config = parse_config(args)
    logger.debug(f"Using configuration: {config!r}")

    elements = load_elements(config["order"], config["include"], config["settings"])

    try:
        start(elements, config["interval"], config["click_events"])
    except Exception:
        logger.exception("Unhandled exception in main loop")
        sys.exit(1)



"""
Configuring swaystatus to do your bidding.

Configuration is defined in a toml file. The first file that exists in the
following list (in order of preference) is used for configuration:

    1. The value of the command line option `--config-file`
    2. `$SWAYSTATUS_CONFIG_FILE`
    3. `<directory>/config.toml` where `<directory>` is the value of the
       command line option `--config-dir`
    4. `$SWAYSTATUS_CONFIG_DIR/config.toml`
    5. `$XDG_CONFIG_HOME/swaystatus/config.toml`
    6. `$HOME/.config/swaystatus/config.toml`

At the very minimum, this configuration file should contain the key to define
elements that will produce content blocks:

    order = ['hostname', 'clock']

If it's not obvious by the name of the key, it also defines the order that
elements will be displayed.

Each name in that list should correspond to a python file contained in a
modules package visible to swaystatus. For a modules package to be visible to
swaystatus, it must be one of the following:

    1. A python package path provided to the command line interface option
       `--include` (can be used multiple times).

    2. A python package called `modules` in the configuration directory. The
       first package that exists in the following list (in order of preference)
       will be visible:

          a. `<directory>/modules/` where `<directory>` is the value of the
             command line option `--config-dir`
          b. `$SWAYSTATUS_CONFIG_DIR/modules/`
          c. `$XDG_CONFIG_HOME/swaystatus/modules/`
          d. `$HOME/.config/swaystatus/modules/`

    3. A python package path specified in the configuration file:

        include = ['/path/to/modules1', '/path/to/modules2']

    4. A python package path specified in an environment variable:

        SWAYSTATUS_MODULE_PATH=/path/to/modules1:/path/to/modules2

    5. An installed python package with an entry point for `swaystatus.modules`
       defined like the following in your `pyproject.toml` (obviously your
       package name will be different):

          [project.entry-points."swaystatus.modules"]
          package = "my_awesome_swaystatus_modules"

Any combination of the above methods can be used. Packages are searched for
modules in the order of preference defined above. If any packages contain
modules with the same name, the first package that provides it will be used.

The following keys are recognized in the configuration file:

    - `order`: A list of module names to use in the status bar. It also defines
      the order in which they will be displayed. Modules can be included more
      than once. Modules can also be defined with an instance name, like
      `module:foo`, which can have their own settings (see the example below).

    - `interval`: A float that dictates how often the status bar will be
      updated (in seconds, default: 1.0).

    - `click_events`: A boolean that dictates whether or not click events will
      be sent to swaystatus by swaybar over stdin (default: true).

    - `include`: A list of directory paths to python packages that will be
      included when searching for module files.

    - `env`: A dictionary that defines environment variables that will be
      available to any click handler shell commands. This can be defined at the
      top level, for a module, or for a specific instance of a module.

    - `on_click`: A dictionary that maps pointer device button numbers to shell
      commands. The commands can be defined as either a single string or a list
      of strings. This can be defined at the top level, for a module, or for a
      specific instance of a module.

    - `settings`: A dictionary that maps module names to dictionaries that will
      be passed to the element's initializer as keyword arguments.

If any key is missing, it's default will be used. The `order` key isn't even
required, but to see anything in the status bar, it should contain at least one
module.

For example, the configuration file might look like:

    order = [
        'hostname',
        'path_exists:/mnt/foo',
        'memory',
        'clock',
        'clock:work'
    ]

    [env]
    terminal = 'foot'

    [settings.hostname]
    full_text = "host: {}"

    [settings.path_exists]
    on_click.1 = '$terminal --working-directory="$instance"'
    on_click.2 = '$terminal --hold df $instance'

    [settings."clock:work".env]
    TZ = 'Asia/Tokyo'
"""


config = {
    "order": [],
    "interval": 1.0,
    "click_events": True,
    "include": [],
    "env": {},
    "on_click": {},
    "settings": {},
}



"""
Generate a status line for swaybar.

This package provides a simple and flexible framework to facilitate the
production of an interactive status line for swaybar.

There are two primary ways of interacting with this package:

1. Defining modules using the base class `swaystatus.element.BaseElement` to
   produce status bar blocks. For details, see the documentation for
   `swaystatus.element`.

2. Producing encoded content for swaybar with the `swaystatus` command. For
   details on the command line interface, run `swaystatus --help`. For details
   on configuring swaystatus, see the documentation for `swaystatus.config`.

This package does not contain any modules. It simply provides a way to make
them usable. One of the goals of this project is to be as unopinionated as
possible. I prefer simple, easy to read, modules, but you may want colorful,
feature-filled, eye candy. This package supports both, but provides neither.

This package does support the usage of external module packages, however,
making it easy to use any number of published module collections. For example,
I might have my modules published on PyPI as `my-awesome-swaystatus-modules`
and as long as that package has an entry point defined for
`swaystatus.modules`, it will be found by swaystatus and its modules available
to use.

Something like the following in the `pyproject.toml` for your modules package
should suffice:

    [project.entry-points."swaystatus.modules"]
    package = 'my_awesome_swaystatus_modules'
"""
